﻿Imports System.IO
Imports System.Text
Imports System.Threading.Tasks

Namespace SplitAndMerge
  Friend Class [Continue]
	  Inherits Parser.Result
  End Class

  Friend Class ContinueStatement
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Return New [Continue]()
	End Function
  End Class

  Friend Class Break
	  Inherits Parser.Result
  End Class

  Friend Class BreakStatement
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Return New Break()
	End Function
  End Class


  Friend Class PiFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Return New Parser.Result(3.141592653589793)
	End Function
  End Class

  Friend Class ExpFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim result As Parser.Result = Parser.LoadAndCalculate(data, [from], Constants.END_ARG_ARRAY)
	  result.Value = Math.Exp(result.Value)
	  Return result
	End Function
  End Class

  Friend Class PowFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim arg1 As Parser.Result = Parser.LoadAndCalculate(data, [from], Constants.NEXT_ARG_ARRAY)
	  [from] += 1 ' eat separation
	  Dim arg2 As Parser.Result = Parser.LoadAndCalculate(data, [from], Constants.END_ARG_ARRAY)

	  arg1.Value = Math.Pow(arg1.Value, arg2.Value)
	  Return arg1
	End Function
  End Class

  Friend Class SinFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim arg As Parser.Result = Parser.LoadAndCalculate(data, [from], Constants.END_ARG_ARRAY)
	  arg.Value = Math.Sin(arg.Value)
	  Return arg
	End Function
  End Class

  Friend Class SqrtFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim arg As Parser.Result = Parser.LoadAndCalculate(data, [from], Constants.END_ARG_ARRAY)
	  arg.Value = Math.Sqrt(arg.Value)
	  Return arg
	End Function
  End Class

  Friend Class AbsFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim arg As Parser.Result = Parser.LoadAndCalculate(data, [from], Constants.END_ARG_ARRAY)
	  arg.Value = Math.Abs(arg.Value)
	  Return arg
	End Function
  End Class

  Friend Class PsInfoFunction
	  Inherits ParserFunction
	Friend Sub New(ByVal interpreter As Interpreter)
	  m_interpreter = interpreter
	End Sub

	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim pattern As String = Utils.GetItem(data, [from]).String
	  If String.IsNullOrWhiteSpace(pattern) Then
		Throw New ArgumentException("Couldn't extract process name")
	  End If

	  Dim MAX_PROC_NAME As Integer = 26
	  m_interpreter.AppendOutput(Utils.GetLine())
	  m_interpreter.AppendOutput(String.Format("{0} {1} {2} {3} {4}", "Process Id".PadRight(15), "Process Name".PadRight(MAX_PROC_NAME), "Working Set".PadRight(15), "Virt Mem".PadRight(15), "CPU Time".PadRight(25)))

	  Dim processes() As Process = Process.GetProcessesByName(pattern)
	  Dim results As New List(Of Parser.Result)(processes.Length)
	  For i As Integer = 0 To processes.Length - 1
		Dim pr As Process = processes(i)
		Dim workingSet As Integer = CInt(Fix((CDbl(pr.WorkingSet64)) / 1000000.0))
		Dim virtMemory As Integer = CInt(Fix((CDbl(pr.VirtualMemorySize64)) / 1000000.0))
		Dim procTitle As String = pr.ProcessName & " " & pr.MainWindowTitle.Split(Nothing)(0)
		If procTitle.Length > MAX_PROC_NAME Then
		  procTitle = procTitle.Substring(0, MAX_PROC_NAME)
		End If
		Dim procTime As String = String.Empty
		Try
		  procTime = pr.TotalProcessorTime.ToString().Substring(0, 11)
		Catch e1 As Exception
		End Try

		results.Add(New Parser.Result(Double.NaN, String.Format("{0,15} {1," & MAX_PROC_NAME & "} {2,15} {3,15} {4,25}", pr.Id, procTitle, workingSet, virtMemory, procTime)))
		m_interpreter.AppendOutput(results.Last().String)
	  Next i
	  m_interpreter.AppendOutput(Utils.GetLine())

	  If data.Length > [from] AndAlso data.Chars([from]) = Constants.NEXT_ARG Then
		[from] += 1 ' eat end of statement semicolon
	  End If

	  Return New Parser.Result(Double.NaN, Nothing, results)
	End Function

	Private m_interpreter As Interpreter
  End Class

  Friend Class KillFunction
	  Inherits ParserFunction
	Friend Sub New(ByVal interpreter As Interpreter)
	  m_interpreter = interpreter
	End Sub

	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim id As Parser.Result = Utils.GetItem(data, [from], True) ' expectInt

	  Dim processId As Integer = CInt(Fix(id.Value))
	  Try
		Dim process As Process = Process.GetProcessById(processId)
		process.Kill()
		m_interpreter.AppendOutput("Process " & processId & " killed")
	  Catch exc As Exception
		Throw New ArgumentException("Couldn't kill process " & processId & " (" & exc.Message & ")")
	  End Try

	  Return New Parser.Result()
	End Function

	Private m_interpreter As Interpreter
  End Class

  Friend Class RunFunction
	  Inherits ParserFunction
	Friend Sub New(ByVal interpreter As Interpreter)
	  m_interpreter = interpreter
	End Sub

	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim processName As String = Utils.GetItem(data, [from]).String

	  If String.IsNullOrWhiteSpace(processName) Then
		Throw New ArgumentException("Couldn't extract process name")
	  End If

	  Dim args As List(Of String) = Utils.GetFunctionArgs(data, [from])
	  Dim processId As Integer = -1
	  Try
		Dim pr As Process = Process.Start(processName, String.Join("", args.ToArray()))
		processId = pr.Id
	  Catch exc As System.ComponentModel.Win32Exception
		Throw New ArgumentException("Couldn't start [" & processName & "]: " & exc.Message)
	  End Try

	  m_interpreter.AppendOutput("Process " & processName & " started, id: " & processId)
	  Return New Parser.Result(processId)
	End Function

	Private m_interpreter As Interpreter
  End Class

  Friend Class GetEnvFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim varName As String = Utils.GetToken(data, [from], Constants.END_ARG_ARRAY)
	  Dim res As String = Environment.GetEnvironmentVariable(varName)

	  Return New Parser.Result(Double.NaN, res)
	End Function
  End Class

  Friend Class SetEnvFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim varName As String = Utils.GetToken(data, [from], Constants.NEXT_ARG_ARRAY)
	  If [from] >= data.Length Then
		Throw New ArgumentException("Couldn't set env variable")
	  End If

	  Dim varValue As Parser.Result = Utils.GetItem(data, [from])
	  Dim strValue As String = If(varValue.String IsNot Nothing, varValue.String, varValue.Value.ToString())
	  Environment.SetEnvironmentVariable(varName, strValue)

	  Return New Parser.Result(Double.NaN, varName)
	End Function
  End Class

  Friend Class PrintFunction
	  Inherits ParserFunction
	Friend Sub New(ByVal interpreter As Interpreter)
	  m_interpreter = interpreter
	End Sub
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim args As List(Of String) = Utils.GetFunctionArgs(data, [from])
	  m_interpreter.AppendOutput(String.Join("", args.ToArray()))

	  Return New Parser.Result()
	End Function
	Private m_interpreter As Interpreter
  End Class

  Friend Class IfStatement
	  Inherits ParserFunction
	Friend Sub New(ByVal interpreter As Interpreter)
	  m_interpreter = interpreter
	End Sub

	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  m_interpreter.CurrentChar = [from]

	  Dim result As Parser.Result = m_interpreter.ProcessIf()

	  Return result
	End Function

	Private m_interpreter As Interpreter
  End Class

  Friend Class WhileStatement
	  Inherits ParserFunction
	Friend Sub New(ByVal interpreter As Interpreter)
	  m_interpreter = interpreter
	End Sub

	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  m_interpreter.CurrentChar = [from]
	  m_interpreter.ProcessWhile()

	  Return New Parser.Result()
	End Function

	Private m_interpreter As Interpreter
  End Class

  Friend Class ProcessorTimeFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim pr As Process = Process.GetCurrentProcess()
	  Dim ts As TimeSpan = pr.TotalProcessorTime

	  Return New Parser.Result(ts.TotalMilliseconds)
	End Function
  End Class

  Friend Class PwdFunction
	  Inherits ParserFunction
	Friend Sub New(ByVal interpreter As Interpreter)
	  m_interpreter = interpreter
	End Sub

	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim path As String = Directory.GetCurrentDirectory()
	  m_interpreter.AppendOutput(path)

	  Return New Parser.Result(Double.NaN, path)
	End Function

	Private m_interpreter As Interpreter
  End Class

  Friend Class Cd__Function
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim newDir As String = Nothing

	  Try
		Dim pwd As String = Directory.GetCurrentDirectory()
		newDir = Directory.GetParent(pwd).FullName
		Directory.SetCurrentDirectory(newDir)
	  Catch exc As Exception
		Throw New ArgumentException("Could not change directory: " & exc.Message)
	  End Try

	  Return New Parser.Result(Double.NaN, newDir)
	End Function
  End Class

  Friend Class CdFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim newDir As String = Utils.GetToken(data, [from], Constants.END_ARG_ARRAY)

	  Try
		Directory.SetCurrentDirectory(newDir)
	  Catch exc As Exception
		Throw New ArgumentException("Could not change directory: " & exc.Message)
	  End Try

	  Return New Parser.Result(Double.NaN, newDir)
	End Function
  End Class

  Friend Class FindstrFunction
	  Inherits ParserFunction
	Friend Sub New(ByVal interpreter As Interpreter)
	  m_interpreter = interpreter
	End Sub

	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim search As String = Utils.GetToken(data, [from], Constants.QUOTE_ARRAY)
	  Dim patterns As List(Of String) = Utils.GetFunctionArgs(data, [from])

	  Dim ignoreCase As Boolean = True
	  If patterns.Count > 0 AndAlso patterns.Last().Equals("case") Then
		ignoreCase = False
		patterns.RemoveAt(patterns.Count - 1)
	  End If
	  If patterns.Count = 0 Then
		patterns.Add("*.*")
	  End If

	  Dim results As List(Of Parser.Result) = Nothing
	  Try
		Dim pwd As String = Directory.GetCurrentDirectory()
		Dim files As List(Of String) = Utils.GetStringInFiles(pwd, search, patterns.ToArray(), ignoreCase)

		m_interpreter.AppendOutput("----- Files containing [" & search & "] with pattern [" & String.Join(", ", patterns.ToArray()) & "] -----")
		results = New List(Of Parser.Result)(files.Count)
		For Each filename As String In files
		  results.Add(New Parser.Result(Double.NaN, filename))
		  m_interpreter.AppendOutput(filename)
		Next filename
		m_interpreter.AppendOutput(Utils.GetLine())
	  Catch exc As Exception
		Throw New ArgumentException("Couldn't find pattern: " & exc.Message)
	  End Try

	  Return New Parser.Result(Double.NaN, Nothing, results)
	End Function

	Private m_interpreter As Interpreter
  End Class

  Friend Class FindfilesFunction
	  Inherits ParserFunction
	Friend Sub New(ByVal interpreter As Interpreter)
	  m_interpreter = interpreter
	End Sub

	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim patterns As List(Of String) = Utils.GetFunctionArgs(data, [from])
	  If patterns.Count = 0 Then
		patterns.Add("*.*")
	  End If

	  Dim results As List(Of Parser.Result) = Nothing
	  Try
		Dim pwd As String = Directory.GetCurrentDirectory()
		Dim files As List(Of String) = Utils.GetFiles(pwd, patterns.ToArray())

		'm_interpreter.AppendOutput("----- Found files with pattern [" +
		'                           string.Join(", ", patterns.ToArray()) + "] -----");
		results = New List(Of Parser.Result)(files.Count)
		For Each filename As String In files
		  results.Add(New Parser.Result(Double.NaN, filename))
		  'm_interpreter.AppendOutput(filename);
		Next filename

		'm_interpreter.AppendOutput(Utils.GetLine());
	  Catch exc As Exception
		Throw New ArgumentException("Couldn't list directory: " & exc.Message)
	  End Try

	  Return New Parser.Result(Double.NaN, Nothing, results)
	End Function

	Private m_interpreter As Interpreter
  End Class

  Friend Class DirFunction
	  Inherits ParserFunction
	Friend Sub New(ByVal interpreter As Interpreter)
	  m_interpreter = interpreter
	End Sub

	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim dirname As String = If(data.Length <= [from] OrElse data.Chars([from]) = Constants.END_STATEMENT, Directory.GetCurrentDirectory(), Utils.GetToken(data, [from], Constants.END_ARG_ARRAY))

	  Try
		Dim results = Directory.EnumerateFiles(dirname)
		m_interpreter.AppendOutput("----- Files in [" & dirname & "] -----")
		For Each filename As String In results
		  m_interpreter.AppendOutput(filename)
		Next filename
		m_interpreter.AppendOutput(Utils.GetLine())
	  Catch exc As Exception
		Throw New ArgumentException("Could not list directory: " & exc.Message)
	  End Try

	  Return New Parser.Result()
	End Function

	Private m_interpreter As Interpreter
  End Class

  Friend Class SetVarFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim varName As String = Utils.GetToken(data, [from], Constants.NEXT_ARG_ARRAY)
	  If [from] >= data.Length Then
		Throw New ArgumentException("Couldn't set variable before end of line")
	  End If

	  Dim varValue As Parser.Result = Utils.GetItem(data, [from])

	  ' Check if the variable to be set has the form of x(0),
	  ' meaning that this is an array element.
	  Dim arrayIndex As Integer = Utils.ExtractArrayElement(varName)
	  If arrayIndex >= 0 Then
		Dim exists As Boolean = ParserFunction.FunctionExists(varName)
		Dim currentValue As Parser.Result = If(exists, ParserFunction.GetFunction(varName).GetValue(data, [from]), New Parser.Result())

		Dim tuple As List(Of Parser.Result) = If(currentValue.Tuple Is Nothing, New List(Of Parser.Result)(), currentValue.Tuple)
		If tuple.Count > arrayIndex Then
		  tuple(arrayIndex) = varValue
		Else
		  For i As Integer = tuple.Count To arrayIndex - 1
			tuple.Add(New Parser.Result(Double.NaN, String.Empty))
		  Next i
		  tuple.Add(varValue)
		End If

		varValue = New Parser.Result(Double.NaN, Nothing, tuple)
	  End If

	  ParserFunction.AddFunction(varName, New GetVarFunction(varValue))

	  Return New Parser.Result(Double.NaN, varName)
	End Function
  End Class

  Friend Class GetVarFunction
	  Inherits ParserFunction
	Friend Sub New(ByVal value As Parser.Result)
	  m_value = value
	End Sub

	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  ' First check if this element is part of an array:
	  If [from] < data.Length AndAlso data.Chars([from] - 1) = Constants.START_ARG Then
		' There is an index given - it may be for an element of the tuple.
		If m_value.Tuple Is Nothing OrElse m_value.Tuple.Count = 0 Then
		  Throw New ArgumentException("No tuple exists for the index")
		End If

		Dim index As Parser.Result = Utils.GetItem(data, [from], True) ' expectInt
		If index.Value < 0 OrElse index.Value >= m_value.Tuple.Count Then
		  Throw New ArgumentException("Incorrect index [" & index.Value & "] for tuple of size " & m_value.Tuple.Count)
		End If
		Return m_value.Tuple(CInt(Fix(index.Value)))
	  End If

	  Return m_value
	End Function

	Private m_value As Parser.Result
  End Class

  Friend Class AppendFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  ' 1. Get the name of the variable.
	  Dim varName As String = Utils.GetToken(data, [from], Constants.NEXT_ARG_ARRAY)
	  If [from] >= data.Length Then
		Throw New ArgumentException("Couldn't append variable")
	  End If

	  ' 2. Get the current value of the variable.
	  Dim func As ParserFunction = ParserFunction.GetFunction(varName)
	  Dim currentValue As Parser.Result = func.GetValue(data, [from])

	  ' 3. Get the value to be added or appended.
	  Dim newValue As Parser.Result = Utils.GetItem(data, [from])

	  ' 4. Take either the string part if it is defined,
	  ' or the numerical part converted to a string otherwise.
	  Dim arg1 As String = If(currentValue.String IsNot Nothing, currentValue.String, currentValue.Value.ToString())
	  Dim arg2 As String = If(newValue.String IsNot Nothing, newValue.String, newValue.Value.ToString())

	  ' 5. The variable becomes a string after adding a string to it.
	  newValue.Reset()
	  newValue.String = arg1 & arg2

	  ParserFunction.AddFunction(varName, New GetVarFunction(newValue))

	  Return newValue
	End Function
  End Class

  Friend Class ToUpperFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  ' 1. Get the name of the variable.
	  Dim varName As String = Utils.GetToken(data, [from], Constants.END_ARG_ARRAY)
	  If [from] >= data.Length Then
		Throw New ArgumentException("Couldn't get variable")
	  End If

	  ' 2. Get the current value of the variable.
	  Dim func As ParserFunction = ParserFunction.GetFunction(varName)
	  Dim currentValue As Parser.Result = func.GetValue(data, [from])

	  ' 3. Take either the string part if it is defined,
	  ' or the numerical part converted to a string otherwise.
	  Dim arg As String = If(currentValue.String IsNot Nothing, currentValue.String, currentValue.Value.ToString())

	  Dim newValue As New Parser.Result(Double.NaN, arg.ToUpper())
	  Return newValue
	End Function
  End Class

  Friend Class ToLowerFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  ' 1. Get the name of the variable.
	  Dim varName As String = Utils.GetToken(data, [from], Constants.END_ARG_ARRAY)
	  If [from] >= data.Length Then
		Throw New ArgumentException("Couldn't get variable")
	  End If

	  ' 2. Get the current value of the variable.
	  Dim func As ParserFunction = ParserFunction.GetFunction(varName)
	  Dim currentValue As Parser.Result = func.GetValue(data, [from])

	  ' 3. Take either the string part if it is defined,
	  ' or the numerical part converted to a string otherwise.
	  Dim arg As String = If(currentValue.String IsNot Nothing, currentValue.String, currentValue.Value.ToString())

	  Dim newValue As New Parser.Result(Double.NaN, arg.ToLower())
	  Return newValue
	End Function
  End Class

  Friend Class SizeFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  ' 1. Get the name of the variable.
	  Dim varName As String = Utils.GetToken(data, [from], Constants.END_ARG_ARRAY)
	  If [from] >= data.Length Then
		Throw New ArgumentException("Couldn't get variable")
	  End If

	  ' 2. Get the current value of the variable.
	  Dim func As ParserFunction = ParserFunction.GetFunction(varName)
	  Dim currentValue As Parser.Result = func.GetValue(data, [from])

	  ' 3. Take either the length of the underlying tuple or
	  ' string part if it is defined,
	  ' or the numerical part converted to a string otherwise.
	  Dim size As Integer = If(currentValue.Tuple IsNot Nothing, currentValue.Tuple.Count, If(currentValue.String IsNot Nothing, currentValue.String.Length, currentValue.Value.ToString().Length))

	  Dim newValue As New Parser.Result(size)
	  Return newValue
	End Function
  End Class

  Friend Class SubstrFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  Dim substring As String

	  ' 1. Get the name of the variable.
	  Dim varName As String = Utils.GetToken(data, [from], Constants.NEXT_ARG_ARRAY)
	  If [from] >= data.Length Then
		Throw New ArgumentException("Couldn't get variable")
	  End If

	  ' 2. Get the current value of the variable.
	  Dim func As ParserFunction = ParserFunction.GetFunction(varName)
	  Dim currentValue As Parser.Result = func.GetValue(data, [from])

	  ' 3. Take either the string part if it is defined,
	  ' or the numerical part converted to a string otherwise.
	  Dim arg As String = If(currentValue.String IsNot Nothing, currentValue.String, currentValue.Value.ToString())
	  ' 4. Get the initial index of the substring.
	  Dim lengthAvailable As Boolean = Utils.SeparatorExists(data, [from])

	  Dim init As Parser.Result = Utils.GetItem(data, [from], True) ' expectInt

	  ' 5. Get the length of the substring if available.
	  If lengthAvailable Then
		Dim length As Parser.Result = Utils.GetItem(data, [from], True) ' expectInt
		If init.Value + length.Value > arg.Length Then
		  Throw New ArgumentException("The total substring length is larger than  [" & arg & "]")
		End If
		substring = arg.Substring(CInt(Fix(init.Value)), CInt(Fix(length.Value)))
	  Else
		substring = arg.Substring(CInt(Fix(init.Value)))
	  End If
	  Dim newValue As New Parser.Result(Double.NaN, substring)

	  Return newValue
	End Function
  End Class

  Friend Class IndexOfFunction
	  Inherits ParserFunction
	Protected Overrides Function Evaluate(ByVal data As String, ByRef [from] As Integer) As Parser.Result
	  ' 1. Get the name of the variable.
	  Dim varName As String = Utils.GetToken(data, [from], Constants.NEXT_ARG_ARRAY)
	  If [from] >= data.Length Then
		Throw New ArgumentException("Couldn't append variable")
	  End If

	  ' 2. Get the current value of the variable.
	  Dim func As ParserFunction = ParserFunction.GetFunction(varName)
	  Dim currentValue As Parser.Result = func.GetValue(data, [from])

	  ' 3. Get the value to be looked for.
	  Dim searchValue As Parser.Result = Utils.GetItem(data, [from])

	  ' 4. Take either the string part if it is defined,
	  ' or the numerical part converted to a string otherwise.
	  Dim basePart As String = If(currentValue.String IsNot Nothing, currentValue.String, currentValue.Value.ToString())
	  Dim search As String = If(searchValue.String IsNot Nothing, searchValue.String, searchValue.Value.ToString())

	  Dim result As Integer = basePart.IndexOf(search)
	  Return New Parser.Result(result, Nothing)
	End Function
  End Class
End Namespace
